@@ -1,13 +1,13 @@
 =head1 NAME
 
-Math::PRBS - Generate Pseudorandom Binary Sequences using an Iterator-based Linear Feedback Shift Register
+Math::PRBS - Generate Pseudorandom Binary Sequences using an iterator-based Linear Feedback Shift Register
 
 =cut
 package Math::PRBS;
 use warnings;
 use strict;
 
-use version 0.77; our $VERSION = version->declare('0.002');
+use version 0.77; our $VERSION = version->declare('0.004');
 
 =head1 SYNOPSIS
 
@@ -16,12 +16,13 @@ use version 0.77; our $VERSION = version->declare('0.002');
     my $prbs7 = Math::PRBS->new( prbs => 7 );
     my ($i, $value) = $x3x2t->next();
     my @p7 = $prbs7->generate_all();
+    my @ints = $prbs7->generate_all_int();
 
 =head1 DESCRIPTION
 
 This module will generate various Pseudorandom Binary Sequences (PRBS).  This module creates a iterator object, and you can use that object to generate the sequence one value at a time, or I<en masse>.
 
-The generated sequence is a series of 0s and 1s which appears random for a certain length, and then repeats thereafter.
+The generated sequence is a series of 0s and 1s which appears random for a certain length, and then repeats thereafter.  (You can also convert the bitstream into a sequence of integers using the C<generate_int> and C<generate_all_int> methods.)
 
 It is implemented using an XOR-based Linear Feedback Shift Register (LFSR), which is described using a feedback polynomial (or reciprocal characteristic polynomial).  The terms that appear in the polynomial are called the 'taps', because you tap off of that bit of the shift register for generating the feedback for the next value in the sequence.
 
@@ -31,7 +32,7 @@ It is implemented using an XOR-based Linear Feedback Shift Register (LFSR), whic
 
 =over
 
-=item C<$seq = Math::PRBS::new( I<key =E<gt> value> )>
+=item C<$seq = Math::PRBS->new( I<key =E<gt> value> )>
 
 Creates the sequence iterator C<$seq> using one of the C<key =E<gt> value> pairs described below.
 
@@ -50,7 +51,7 @@ sub new {
 C<prbs> needs an integer I<n> to indicate one of the "standard" PRBS polynomials.
 
     # example: PRBS7 = x**7 + x**6 + 1
-    $seq = Math::PRBS::new( ptbs => 7 );
+    $seq = Math::PRBS->new( prbs => 7 );
 
 The "standard" PRBS polynomials implemented are
 
@@ -71,7 +72,7 @@ The "standard" PRBS polynomials implemented are
             23 => [23,18],
             31 => [31,28],
         );
-        die __PACKAGE__."::new(prbs => '$pairs{prbs}'): standard PRBS include 7, 15, 23, 31" unless exists $prbs{ $pairs{prbs} };
+        die __PACKAGE__."->new(prbs => '$pairs{prbs}'): standard PRBS include 7, 15, 23, 31" unless exists $prbs{ $pairs{prbs} };
         $self->{taps} = [ @{ $prbs{ $pairs{prbs} } } ];
     }
 
@@ -81,15 +82,15 @@ C<taps> needs an array reference containing the powers in the polynomial that yo
 
     # example: x**3 + x**2 + 1
     #   3 and 2 are taps, 1 is not tapped, 0 is implied feedback
-    $seq = Math::PRBS::new( taps => [3,2] );
+    $seq = Math::PRBS->new( taps => [3,2] );
 
 =cut
 
     elsif( exists $pairs{taps} )
     {
-        die __PACKAGE__."::new(taps => $pairs{taps}): argument should be an array reference" unless 'ARRAY' eq ref($pairs{taps});
+        die __PACKAGE__."->new(taps => $pairs{taps}): argument should be an array reference" unless 'ARRAY' eq ref($pairs{taps});
         $self->{taps} = [ sort {$b <=> $a} @{ $pairs{taps} } ];     # taps in descending order
-        die __PACKAGE__."::new(taps => [@{$pairs{taps}}]): need at least one tap" unless @{ $pairs{taps} };
+        die __PACKAGE__."->new(taps => [@{$pairs{taps}}]): need at least one tap" unless @{ $pairs{taps} };
     }
 
 =item C<poly =E<gt> '...'>
@@ -98,23 +99,23 @@ C<poly> needs a string for the bits C<x**k> downto C<x**1>, with a 1 indicating
 
     # example: x**3 + x**2 + 1
     #   3 and 2 are taps, 1 is not tapped, 0 is implied feedback
-    $seq = Math::PRBS::new( poly => '110' );
+    $seq = Math::PRBS->new( poly => '110' );
 
 =cut
 
     elsif( exists $pairs{poly} )
     {
         local $_ = $pairs{poly};    # used for implicit matching in die-unless and while-condition
-        die __PACKAGE__."::new(poly => '$pairs{poly}'): argument should be an binary string" unless /^[01]*$/;
+        die __PACKAGE__."->new(poly => '$pairs{poly}'): argument should be an binary string" unless /^[01]*$/;
         my @taps = ();
         my $l = length;
         while( m/([01])/g ) {
             push @taps, $l - pos() + 1     if $1;
         }
         $self->{taps} = [ reverse sort {$a <=> $b} @taps ];
-        die __PACKAGE__."::new(poly => '$pairs{poly}'): need at least one tap" unless @taps;
+        die __PACKAGE__."->new(poly => '$pairs{poly}'): need at least one tap" unless @taps;
     } else {
-        die __PACKAGE__."::new(".join(',',@_)."): unknown arguments";
+        die __PACKAGE__."->new(".join(',',@_)."): unknown arguments";
     }
 
     $self->{lfsr} = oct('0b1' . '0'x($self->{taps}[0] - 1));
@@ -261,7 +262,7 @@ If C<limit => $n> is used, will not seek beyond C<tell_i == $n>.
 sub seek_to_end {
     my $self = shift;
 
-    die __PACKAGE__."::generate_to_end(@_) requires even number of arguments, expecting name=>value pairs" unless 0 == @_ % 2;
+    die __PACKAGE__."::seek_to_end(@_) requires even number of arguments, expecting name=>value pairs" unless 0 == @_ % 2;
 
     my %opts = map lc, @_;  # lowercase name,value pairs for canonical
     my $limit = exists $opts{limit} ? $opts{limit} : 65535;
@@ -336,6 +337,42 @@ sub generate_all {
     return generate_to_end($self, %opts);
 }
 
+=item C<@all = $seq-E<gt>generate_int( I<$n> )>
+
+Generates the next I<$n> integers in the sequence, wrapping around if it reaches the end (it will generate just one if I<$n> is missing).  In list context, returns the values as a list; in scalar context, returns the string concatenating that list.
+
+=cut
+
+sub generate_int {
+    my ($self, $n) = @_;
+    $n = 1 unless defined $n;
+    my $k = $self->k_bits();
+    my @arr = map { oct '0b' . $self->generate($k) } 1 .. $n;
+    return wantarray ? @arr : join ',', @arr;
+}
+
+=item C<@all = $seq-E<gt>generate_all_int( )>
+
+=item C<@all = $seq-E<gt>generate_all_int( I<limit =E<gt> $max_i> )>
+
+Returns the whole sequence of C<k>-bit integers, from the beginning, up to the end of the sequence; in list context, returns the list of values; in scalar context, returns the string concatenating that list.  If the sequence is longer than the default limit of 65535, or the limit given by C<$max_i> if the optional C<limit =E<gt> $max_i> is provided, then it will stop before the end of the sequence.
+
+=cut
+
+sub generate_all_int {
+    my $self = shift;
+
+    die __PACKAGE__."::generate_all_int(@_) requires even number of arguments, expecting name=>value pairs" unless 0 == @_ % 2;
+
+    my %opts = map lc, @_;  # lowercase name,value pairs for canonical
+    my $limit = exists $opts{limit} ? $opts{limit} : 65535;
+    my $maxlimit = 2 ** $self->k_bits - 1;
+    my $period = $self->period( force => $maxlimit );
+    $limit = $period if lc($limit) eq 'max' or $limit > $period;
+    $self->rewind();
+    return $self->generate_int( $limit );
+}
+
 =back
 
 =head2 Information
@@ -360,6 +397,29 @@ sub description {
     return "PRBS from polynomial $p + 1";
 }
 
+=item C<$n = $seq-E<gt>polynomial_degree>
+
+=item C<$n = $seq-E<gt>k_bits>
+
+Returns the highest power C<k> from the PRBS polynomial.  As described in the L</Theory> section, if you group a maximum length sequence sequence into groups of C<k> bits, you will produce all the C<k>-bit numbers from C<1> to C<2**k - 1>.
+
+    $seq = Math::PRBS->new( taps => [6,7] );
+    $k = $seq->k_bits();                     # 7
+
+When using C<generate_int()> to generate the next integer in the sequence, it is consuming C<k_bits()> bits from the sequence to create the decimal integer.
+
+    @integers = $seq->generate_int($num_ints);       # consumes $seq->k_bits() bits of the sequence per integer generated
+
+=cut
+
+sub polynomial_degree {
+    my $self = shift;
+    $self->{degree} = (sort {$b<=>$a} @{ $self->{taps}})[0] unless exists $self->{degree};
+    return $self->{degree};
+}
+
+BEGIN { *k_bits = \&polynomial_degree; } # alias
+
 =item C<$i = $seq-E<gt>taps>
 
 Returns an array-reference containing the list of tap identifiers, which could then be passed to C<-E<gt>new(taps =E<gt> ...)>.
@@ -370,8 +430,7 @@ Returns an array-reference containing the list of tap identifiers, which could t
 =cut
 
 sub taps {
-    my @taps = @{ $_[0]->{taps} };
-    return [@taps];
+    return [@{ $_[0]->{taps} }];
 }
 
 =item C<$i = $seq-E<gt>period( I<force =E<gt> 'estimate' | $n | 'max'> )>
@@ -627,20 +686,22 @@ A pseudorandom binary sequence (PRBS) is the sequence of N unique bits, in this
 
 In an LFSR, the polynomial description (like C<x**3 + x**2 + 1>) indicates which bits are "tapped" to create the feedback bit: the taps are the powers of x in the polynomial (3 and 2).  The C<1> is really the C<x**0> term, and isn't a "tap", in the sense that it isn't used for generating the feedback; instead, that is the location where the new feedback bit comes back into the shift register; the C<1> is in all characteristic polynomials, and is implied when creating a new instance of B<Math::PRBS>.
 
-If the largest power of the polynomial is C<k>, there are C<k+1> bits in the register (one for each of the powers C<k..1> and one for the C<x**0 = 1>'s feedback bit).  For any given C<k>, the largest sequence that can be produced is C<N = 2^k - 1>, and that sequence is called a maximum length sequence  or m-sequence; there can be more than one m-sequence for a given C<k>.  One useful feature of an m-sequence is that if you divide it into every possible partial sequence that's C<k> bits long (wraping from N-1 to 0 to make the last few partial sequences also C<k> bits), you will generate every possible combination of C<k> bits (*), except for C<k> zeroes in a row.  For example,
+If the largest power of the polynomial is C<k> (ie, a polynomial of degree C<k>), there are C<k+1> bits in the register (one for each of the powers C<k> down to C<1> and one for the C<x**0 = 1>'s feedback bit).  For any given C<k>, the largest sequence that can be produced is C<N = 2^k - 1>, and any sequence with that length is called a "maximum length sequence" or m-sequence; there can be more than one m-sequence for a given C<k>.
+
+One useful feature of an m-sequence is that if you divide it into every possible partial sequence that's C<k> bits long (wraping from N-1 to 0 to make the last few partial sequences also C<k> bits), you will generate every possible combination of C<k> bits (*), except for C<k> zeroes in a row.  (It then includes the binary representation of every k-bit integer from C<1> to C<2**k - 1>.)  For example,
 
     # x**3 + x**2 + 1 = "1011100"
-    "_101_1100 " -> 101
-    "1_011_100 " -> 011
-    "10_111_00 " -> 111
-    "101_110_0 " -> 110
-    "1011_100_ " -> 100
-    "1_0111_00 " -> 001 (requires wrap to get three digits: 00 from the end, and 1 from the beginning)
-    "10_1110_0 " -> 010 (requires wrap to get three digits: 0 from the end, and 10 from the beginning)
+    "_101_1100 " -> 101 (5)
+    "1_011_100 " -> 011 (3)
+    "10_111_00 " -> 111 (7)
+    "101_110_0 " -> 110 (6)
+    "1011_100_ " -> 100 (4)
+    "1_0111_00 " -> 001 (1) (requires wrap to get three digits: 00 from the end of the sequence, and 1 from the beginning)
+    "10_1110_0 " -> 010 (2) (requires wrap to get three digits: 0 from the end of the sequence, and 10 from the beginning)
 
-The Wikipedia:LFSR article (see L</REFERENCES>) lists some polynomials that create m-sequence for various register sizes, and links to Philip Koopman's complete list up to C<k=64>.
+The Wikipedia:LFSR article lists some polynomials that create m-sequence for various register sizes, and links to Philip Koopman's complete list up to C<k=64>  (see L</REFERENCES> for links to both).
 
-If you want to create try own polynonial to find a long m-sequence, here are some things to consider: 1) the number of taps for the feedback (remembering not to count the feedback bit as a tap) must be even; 2) the entire set of taps must be relatively prime; 3) those two conditions are necesssary, but not sufficient, so you may have to try multiple polynomials to find an m-sequence; 4) keep in mind that the time to compute the period (and thus determine if it's an m-sequence) doubles every time C<k> increases by 1; as the time increases, it makes more sense to look at the complete list up to C<k=64>), and pure-perl is probably tpp wrong language for searching C<kE<gt>64>.
+If you want to create your own polynonial to find a long m-sequence, here are some things to consider: 1) the number of taps for the feedback (remembering not to count the feedback bit as a tap) must be even; 2) the entire set of taps must be relatively prime; 3) those two conditions are necesssary, but not sufficient, so you may have to try multiple polynomials to find an m-sequence; 4) keep in mind that the time to compute the period (and thus determine if it's an m-sequence) doubles every time C<k> increases by 1; as the time increases, it makes more sense to look at the complete list up to C<k=64>), and pure-perl is probably tpp wrong language for searching C<kE<gt>64>.
 
 (*) Since a maximum length sequence contains every k-bit combination (except all zeroes), it can be used for verifying that software or hardware behaves properly for every possible sequence of k-bits.
 
@@ -676,6 +737,33 @@ If you want to create try own polynonial to find a long m-sequence, here are som
 
 =back
 
+=head1 INSTALLATION
+
+To install this module, use your favorite CPAN client.
+
+For a manual install, type the following:
+
+    perl Makefile.PL
+    make
+    make test
+    make install
+
+(On Windows machines, you may need to use "dmake" or "gmake" instead of "make", depending on your setup.)
+
+=head1 SEE ALSO
+
+=over
+
+=item * L<Math::NumSeq> - an iterator-based sequence generator, with a variety of numeric sequences that can be generated
+
+The primary API for B<Math::PRBS> was based on L<Math::NumSeq>, but it was decided to not make B<Math::PRBS> dependent on it, because L<Math::NumSeq> does not L<seem to install on Windows|http://matrix.cpantesters.org/?dist=Math-NumSeq>.
+
+=item * L<Math::Sequence> and L<Math::Series> - generate sequences using symbolic forumla
+
+=item * L<Math::PSRG> - Implements a (159, 31, 0) LFSR
+
+=back
+
 =head1 AUTHOR
 
 Peter C. Jones C<E<lt>petercj AT cpan DOT orgE<gt>>
@@ -683,11 +771,23 @@ Peter C. Jones C<E<lt>petercj AT cpan DOT orgE<gt>>
 Please report any bugs or feature requests thru the web interface at
 L<https://github.com/pryrt/Math-PRBS/issues>
 
+=begin html
+
+<a href="https://metacpan.org/pod/Math::PRBS"><img src="https://img.shields.io/cpan/v/Math-PRBS.svg?colorB=00CC00" alt="" title="metacpan"></a>
+<a href="http://matrix.cpantesters.org/?dist=Math-PRBS"><img src="http://cpants.cpanauthors.org/dist/Math-PRBS.png" alt="" title="cpan testers"></a>
+<a href="https://github.com/pryrt/Math-PRBS/releases"><img src="https://img.shields.io/github/release/pryrt/Math-PRBS.svg" alt="" title="github release"></a>
+<a href="https://github.com/pryrt/Math-PRBS/issues"><img src="https://img.shields.io/github/issues/pryrt/Math-PRBS.svg" alt="" title="issues"></a>
+<a href="https://ci.appveyor.com/project/pryrt/math-prbs"><img src="https://ci.appveyor.com/api/projects/status/cj6cbq7u9velb8wx?svg=true" alt="" title="appveyor build status"></a>
+<a href="https://travis-ci.org/pryrt/Math-PRBS"><img src="https://travis-ci.org/pryrt/Math-PRBS.svg?branch=master" alt="" title="travis build status"></a>
+<a href="https://coveralls.io/github/pryrt/Math-PRBS?branch=master"><img src="https://coveralls.io/repos/github/pryrt/Math-PRBS/badge.svg?branch=master" alt="" title="test coverage"></a>
+
+=end html
+
 =head1 COPYRIGHT
 
-Copyright (C) 2016 Peter C. Jones
+Copyright (C) 2016,2018 Peter C. Jones
 
-=head1 LICENCE
+=head1 LICENSE
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
